;; Generated package description from .el  -*- elisp-benchmarksno-byte-compile: t -*-
(define-package "elisp-benchmarks" "1.16.0.20250413.231914" "elisp benchmarks collection" 'nil :commit "e2780d406e7a51162f0584f18192733898363330" :url "https://elpa.gnu.org/packages/elisp-benchmarks.html" :authors '(("Andrea Corallo" . "acorallo@gnu.org")) :maintainer '("Andrea Corallo" . "acorallo@gnu.org") :keywords '("languages" "lisp"))
